#include <errno.h>
#include "netex.h"
#include "stdlib.h"

#include "list.h"

list_node * insert_node(list_node* list, void* data)
{
    list_node* node = (list_node*)malloc(sizeof(list_node));
    if (node == NULL)
    {
        int error = errno;
        //Failed -> log
        return NULL;
    }
    node->data = data;
    node->previous = NULL;
    node->next = NULL;

    if (list != NULL)
    {
        if (list->next == NULL)
            list->next = node;
        list_node* current = list->next;
        for (;;)
        {
            if (current->next == NULL)
            {
                current->next = node;
                node->previous = current;
                break;
            }
            current = current->next;
        }
    }

    return node;
}

void delete_node(list_node* node)
{
    if (node == NULL)
        return;
    if (node->previous != NULL)
        node->previous->next = node->next;
    if (node->next != NULL)
        node->next->previous = node->previous;
    free(node);
}

void iterate_list(list_node* list, bool forward, int (*iterFunc)(const void*))
{
    if (list == NULL || iterFunc == NULL)
        return;
    list_node * current = list;
    while (current != NULL)
    {
        if (iterFunc(current->data) == 1)
            break;
        if (forward)
            current = current->next;
        else
            current = current->previous;
    }
}

list_node* find_node(list_node* list_start, bool forward, int (*findFunc)(const void*))
{
    if (list_start == NULL || findFunc == NULL)
        return NULL;
    list_node* current = list_start;
    while (current != NULL)
    {
        if (findFunc(current->data) == 1)
            return current;
        if (forward)
            current = current->next;
        else
            current = current->previous;
    }
    return NULL;
}